import { CommonModule } from '@angular/common';
import { takeUntil } from 'rxjs/operators';
import { __spread } from 'tslib';
import { EventEmitter, Injectable, ChangeDetectorRef, Directive, Input, ViewContainerRef, NgModule } from '@angular/core';
import { isObservable, Subject } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxBindInputsService = /** @class */ (function () {
    function NgxBindInputsService() {
    }
    /**
     * BindInputs Directive
     */
    /**
     * BindInputs Directive
     * @param {?} directive
     * @return {?}
     */
    NgxBindInputsService.prototype.bindInputs = /**
     * BindInputs Directive
     * @param {?} directive
     * @return {?}
     */
    function (directive) {
        var _this = this;
        /** @type {?} */
        var excludeInputs = (Array.isArray(directive.excludeInputs)
            ? directive.excludeInputs
            : [directive.excludeInputs]).map(function (key) { return key.toUpperCase(); });
        /** @type {?} */
        var includeInputs = (Array.isArray(directive.includeInputs)
            ? directive.includeInputs
            : [directive.includeInputs]).map(function (key) { return key.toUpperCase(); });
        directive.inputs.parentKeys
            .filter(function (parentKey) {
            return (includeInputs.length === 0 && excludeInputs.indexOf(parentKey.toUpperCase()) === -1) ||
                (includeInputs.length !== 0 && includeInputs.indexOf(parentKey.toUpperCase()) !== -1);
        })
            .forEach(function (parentKey) {
            directive.inputs.keys.forEach(function (key) {
                if (_this.checkInputToBind(directive, parentKey, key)) {
                    _this.bindInput(directive, parentKey, key);
                }
            });
        });
    };
    /**
     * @param {?} directive
     * @return {?}
     */
    NgxBindInputsService.prototype.bindObservableInputs = /**
     * @param {?} directive
     * @return {?}
     */
    function (directive) {
        var _this = this;
        /** @type {?} */
        var excludeInputs = (Array.isArray(directive.excludeInputs)
            ? directive.excludeInputs
            : [directive.excludeInputs]).map(function (key) { return key.toUpperCase(); });
        /** @type {?} */
        var includeInputs = (Array.isArray(directive.includeInputs)
            ? directive.includeInputs
            : [directive.includeInputs]).map(function (key) { return key.toUpperCase(); });
        directive.inputs.parentKeys
            .filter(function (parentKey) {
            return (includeInputs.length === 0 && excludeInputs.indexOf(parentKey.toUpperCase()) === -1) ||
                (includeInputs.length !== 0 && includeInputs.indexOf(parentKey.toUpperCase()) !== -1);
        })
            .forEach(function (parentKey) {
            directive.inputs.keys.forEach(function (key) {
                if (_this.checkObservableInputToBind(directive, parentKey, key)) {
                    _this.bindObservableInput(directive, parentKey, key);
                }
            });
        });
    };
    /**
     * Inputs
     */
    /**
     * Inputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindInputsService.prototype.checkKeyNameToInputBind = /**
     * Inputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        return parentKey === key;
    };
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindInputsService.prototype.checkInputToBind = /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        return (directive.used.indexOf(parentKey) === -1 &&
            this.checkKeyNameToInputBind(directive, parentKey, key) &&
            !isObservable(directive.parentComponent[parentKey]) &&
            !isObservable(directive.component[key]));
    };
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindInputsService.prototype.bindInput = /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        directive.used.push(parentKey);
        directive.component[key] = directive.parentComponent[parentKey];
    };
    /**
     * Observable Inputs
     */
    /**
     * Observable Inputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindInputsService.prototype.checkKeyNameToObservableInputBind = /**
     * Observable Inputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        return parentKey === key + "$";
    };
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindInputsService.prototype.checkObservableInputToBind = /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        return (directive.used.indexOf(parentKey) === -1 &&
            this.checkKeyNameToObservableInputBind(directive, parentKey, key) &&
            isObservable(directive.parentComponent[parentKey]) &&
            !isObservable(directive.component[key]));
    };
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindInputsService.prototype.bindObservableInput = /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        directive.used.push(parentKey);
        directive.parentComponent[parentKey]
            .pipe(takeUntil(directive.destroyed$))
            .subscribe(function (value) { return directive.bindValue(key, value); });
    };
    /**
     * Utils
     */
    /**
     * Utils
     * @param {?} directive
     * @return {?}
     */
    NgxBindInputsService.prototype.getInputs = /**
     * Utils
     * @param {?} directive
     * @return {?}
     */
    function (directive) {
        /** @type {?} */
        var data = {
            parentKeys: __spread(Object.keys(directive.parentComponent), Object.keys(directive.parentComponent.__proto__), Object.keys(directive.parentComponent.__proto__ ? directive.parentComponent.__proto__.__proto__ : [])),
            keys: __spread(Object.keys(directive.component ? directive.component : []).filter(function (key) { return !(directive.component[key] instanceof EventEmitter); }))
        };
        return data;
    };
    NgxBindInputsService.decorators = [
        { type: Injectable }
    ];
    return NgxBindInputsService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BindInputsDirective = /** @class */ (function () {
    function BindInputsDirective(_viewContainerRef, _ngxBindInputsService, _ref) {
        this._viewContainerRef = _viewContainerRef;
        this._ngxBindInputsService = _ngxBindInputsService;
        this._ref = _ref;
        this.excludeInputs = [];
        this.includeInputs = [];
        this.used = [];
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    BindInputsDirective.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        this.component = this._viewContainerRef['_data'].componentView.component;
        this.parentComponent = ((/** @type {?} */ (this._viewContainerRef)))._view.context;
        this.inputs = this._ngxBindInputsService.getInputs(this);
        this._ngxBindInputsService.bindInputs(this);
        this._ngxBindInputsService.bindObservableInputs(this);
        if (localStorage && localStorage.getItem('debug_ngx-bind-io') === 'true') {
            console.log(this.inputs, this);
        }
    };
    /**
     * @return {?}
     */
    BindInputsDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    };
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    BindInputsDirective.prototype.bindValue = /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    function (key, value) {
        this.component[key] = value;
        this._ref.markForCheck();
    };
    BindInputsDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[bindInputs]'
                },] }
    ];
    /** @nocollapse */
    BindInputsDirective.ctorParameters = function () { return [
        { type: ViewContainerRef },
        { type: NgxBindInputsService },
        { type: ChangeDetectorRef }
    ]; };
    BindInputsDirective.propDecorators = {
        excludeInputs: [{ type: Input }],
        includeInputs: [{ type: Input }]
    };
    return BindInputsDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} functionToCheck
 * @return {?}
 */
function isFunction(functionToCheck) {
    return functionToCheck && {}.toString.call(functionToCheck) === '[object Function]';
}
/**
 * @param {?} derivedCtor
 * @param {?} baseCtors
 * @return {?}
 */
function applyMixins(derivedCtor, baseCtors) {
    baseCtors.forEach(function (baseCtor) {
        Object.getOwnPropertyNames(baseCtor.prototype).forEach(function (name) {
            derivedCtor.prototype[name] = baseCtor.prototype[name];
        });
    });
    return derivedCtor;
}
/**
 * @param {?} baseCtors
 * @return {?}
 */
function Mixin(baseCtors) {
    return function (derivedCtor) {
        baseCtors.forEach(function (baseCtor) {
            Object.getOwnPropertyNames(baseCtor.prototype).forEach(function (name) {
                derivedCtor.prototype[name] = baseCtor.prototype[name];
            });
        });
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxBindOutputsService = /** @class */ (function () {
    function NgxBindOutputsService() {
    }
    /**
     * BindOutputs Directive
     */
    /**
     * BindOutputs Directive
     * @param {?} directive
     * @return {?}
     */
    NgxBindOutputsService.prototype.bindOutputs = /**
     * BindOutputs Directive
     * @param {?} directive
     * @return {?}
     */
    function (directive) {
        var _this = this;
        /** @type {?} */
        var excludeOutputs = (Array.isArray(directive.excludeOutputs)
            ? directive.excludeOutputs
            : [directive.excludeOutputs]).map(function (key) { return key.toUpperCase(); });
        /** @type {?} */
        var includeOutputs = (Array.isArray(directive.includeOutputs)
            ? directive.includeOutputs
            : [directive.includeOutputs]).map(function (key) { return key.toUpperCase(); });
        directive.outputs.parentKeys
            .filter(function (parentKey) {
            return (includeOutputs.length === 0 && excludeOutputs.indexOf(parentKey.toUpperCase()) === -1) ||
                (includeOutputs.length !== 0 && includeOutputs.indexOf(parentKey.toUpperCase()) !== -1);
        })
            .forEach(function (parentKey) {
            directive.outputs.keys.forEach(function (key) {
                if (_this.checkOutputToBind(directive, parentKey, key)) {
                    _this.bindOutput(directive, parentKey, key);
                }
            });
        });
    };
    /**
     * Outputs
     */
    /**
     * Outputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindOutputsService.prototype.checkKeyNameToOutputBind = /**
     * Outputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        /** @type {?} */
        var keyWithFirstUpperLetter = key.length > 0 ? key.charAt(0).toUpperCase() + key.substr(1) : key;
        return ((parentKey === "on" + keyWithFirstUpperLetter &&
            directive.outputs.parentKeys.indexOf("on" + keyWithFirstUpperLetter + "Click") === -1) ||
            parentKey === "on" + keyWithFirstUpperLetter + "Click");
    };
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindOutputsService.prototype.checkOutputToBind = /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        return directive.used.indexOf(key) === -1 && this.checkKeyNameToOutputBind(directive, parentKey, key);
    };
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    NgxBindOutputsService.prototype.bindOutput = /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    function (directive, parentKey, key) {
        directive.used.push(key);
        directive.component[key].subscribe(function (value) { return directive.parentComponent[parentKey](value); });
    };
    /**
     * Utils
     */
    /**
     * Utils
     * @param {?} directive
     * @return {?}
     */
    NgxBindOutputsService.prototype.getOutputs = /**
     * Utils
     * @param {?} directive
     * @return {?}
     */
    function (directive) {
        /** @type {?} */
        var data = {
            parentKeys: __spread(Object.keys(directive.parentComponent).filter(function (parentKey) { return isFunction(directive.parentComponent[parentKey]); }), Object.keys(directive.parentComponent.__proto__).filter(function (parentKey) {
                return isFunction(directive.parentComponent.__proto__[parentKey]);
            }), Object.keys(directive.parentComponent.__proto__ ? directive.parentComponent.__proto__.__proto__ : []).filter(function (parentKey) { return isFunction(directive.parentComponent.__proto__.__proto__[parentKey]); })),
            keys: __spread(Object.keys(directive.component ? directive.component : []).filter(function (key) { return directive.component[key] instanceof EventEmitter; }))
        };
        return data;
    };
    NgxBindOutputsService.decorators = [
        { type: Injectable }
    ];
    return NgxBindOutputsService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BindIODirective = /** @class */ (function () {
    function BindIODirective(_viewContainerRef, _ngxBindInputsService, _ngxBindOutputsService, _ref) {
        this._viewContainerRef = _viewContainerRef;
        this._ngxBindInputsService = _ngxBindInputsService;
        this._ngxBindOutputsService = _ngxBindOutputsService;
        this._ref = _ref;
        this.excludeInputs = [];
        this.includeInputs = [];
        this.excludeOutputs = [];
        this.includeOutputs = [];
        this.used = [];
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    BindIODirective.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        this.component = this._viewContainerRef['_data'].componentView.component;
        this.parentComponent = ((/** @type {?} */ (this._viewContainerRef)))._view.context;
        this.inputs = this._ngxBindInputsService.getInputs(this);
        this.outputs = this._ngxBindOutputsService.getOutputs(this);
        this._ngxBindInputsService.bindInputs(this);
        this._ngxBindInputsService.bindObservableInputs(this);
        this._ngxBindOutputsService.bindOutputs(this);
        if (localStorage && localStorage.getItem('debug_ngx-bind-io') === 'true') {
            console.log(this.inputs, this.outputs, this);
        }
    };
    /**
     * @return {?}
     */
    BindIODirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    };
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    BindIODirective.prototype.bindValue = /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    function (key, value) {
        this.component[key] = value;
        this._ref.markForCheck();
    };
    BindIODirective.decorators = [
        { type: Directive, args: [{
                    selector: '[bindIO]'
                },] }
    ];
    /** @nocollapse */
    BindIODirective.ctorParameters = function () { return [
        { type: ViewContainerRef },
        { type: NgxBindInputsService },
        { type: NgxBindOutputsService },
        { type: ChangeDetectorRef }
    ]; };
    BindIODirective.propDecorators = {
        excludeInputs: [{ type: Input }],
        includeInputs: [{ type: Input }],
        excludeOutputs: [{ type: Input }],
        includeOutputs: [{ type: Input }]
    };
    return BindIODirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BindOutputsDirective = /** @class */ (function () {
    function BindOutputsDirective(_viewContainerRef, _ngxBindOutputsService, _ref) {
        this._viewContainerRef = _viewContainerRef;
        this._ngxBindOutputsService = _ngxBindOutputsService;
        this._ref = _ref;
        this.excludeOutputs = [];
        this.includeOutputs = [];
        this.used = [];
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    BindOutputsDirective.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        this.component = this._viewContainerRef['_data'].componentView.component;
        this.parentComponent = ((/** @type {?} */ (this._viewContainerRef)))._view.context;
        this.outputs = this._ngxBindOutputsService.getOutputs(this);
        this._ngxBindOutputsService.bindOutputs(this);
        if (localStorage && localStorage.getItem('debug_ngx-bind-io') === 'true') {
            console.log(this.outputs, this);
        }
    };
    /**
     * @return {?}
     */
    BindOutputsDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    };
    BindOutputsDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[bindOutputs]'
                },] }
    ];
    /** @nocollapse */
    BindOutputsDirective.ctorParameters = function () { return [
        { type: ViewContainerRef },
        { type: NgxBindOutputsService },
        { type: ChangeDetectorRef }
    ]; };
    BindOutputsDirective.propDecorators = {
        excludeOutputs: [{ type: Input }],
        includeOutputs: [{ type: Input }]
    };
    return BindOutputsDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxBindIOModule = /** @class */ (function () {
    function NgxBindIOModule() {
    }
    /**
     * @return {?}
     */
    NgxBindIOModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: NgxBindIOModule,
            providers: [NgxBindInputsService, NgxBindOutputsService]
        };
    };
    NgxBindIOModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [BindInputsDirective, BindOutputsDirective, BindIODirective],
                    exports: [BindInputsDirective, BindOutputsDirective, BindIODirective]
                },] }
    ];
    return NgxBindIOModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxBindIOModule, BindInputsDirective, BindIODirective, BindOutputsDirective, NgxBindInputsService, NgxBindOutputsService, isFunction, applyMixins, Mixin };

//# sourceMappingURL=ngx-bind-io.js.map